/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var AddSite = 
{
	edit: false,

	onLoad: function()
	{
		window.sizeToContent();
		ABPPAUtils.E("delay").setAttribute("checked", ABPPA.prefs.closeAsync);

		if(window.arguments.length)
		{
			var rule = window.arguments[0];
			this.edit = window.arguments[1];
			var showRemove = window.arguments[2];

			if(!showRemove)
			{
				document.documentElement.getButton("extra1").hidden = true;
			}

			if(rule.isValid)
			{
				if(this.edit)
				{
					document.title = ABPPAUtils.E("abppa-string-bundle").getString("editTitle");
				}

				ABPPAUtils.E("site").value = rule.site;
				ABPPAUtils.E("delay").setAttribute("checked", rule.closeAsync);

				var actionElement = ABPPAUtils.E("type");
				var winElement = ABPPAUtils.E("windows");
				var tabElement = ABPPAUtils.E("tabs");
				var regElement = ABPPAUtils.E("regular");
				var alertElement = ABPPAUtils.E("alerts");

				winElement.checked = false;
				tabElement.checked = false;
				
				if(rule.hasWindowType(SiteRule.CHECK_WINDOWS))
				{
					winElement.checked = true;
				}
				if(rule.hasWindowType(SiteRule.CHECK_TABS))
				{
					tabElement.checked = true;
				}
				if(rule.hasWindowType(SiteRule.CHECK_REGULAR))
				{
					regElement.checked = true;
				}
				if(rule.hasWindowType(SiteRule.CHECK_DIALOGS))
				{
					alertElement.checked = true;
				}
				
				switch(rule.action)
				{
					case SiteRule.ALLOW:
						actionElement.value = "allow";
						break;
					case SiteRule.BLOCK:
						actionElement.value = "block";
						break;
					case SiteRule.CHECK:
						actionElement.value = "check";
						break;
				}
			}
		}

		this.onActionChanged();
	},

	add: function()
	{
		var site = ABPPAUtils.E("site").value;
		site = site.replace(/^https?:\/\//, "");

		var pos = site.indexOf('/');

		if(pos >= 0)
		{
			site = site.substring(0, pos);
		}

		site = site.replace(/([;\:|]+)/g, '');
		site = site.toLowerCase();

		if(site.length)
		{
			var rule = window.arguments[0];

			var action = ABPPAUtils.E("type").value;
			rule.winTypes = 0;

			if(ABPPAUtils.E("windows").checked)
			{
				rule.addWindowType(SiteRule.CHECK_WINDOWS);
			}
			if(ABPPAUtils.E("tabs").checked)
			{
				rule.addWindowType(SiteRule.CHECK_TABS);
			}
			if(ABPPAUtils.E("regular").checked)
			{
				rule.addWindowType(SiteRule.CHECK_REGULAR);
			}
			if(ABPPAUtils.E("alerts").checked)
			{
				rule.addWindowType(SiteRule.CHECK_DIALOGS);
			}

			switch(action)
			{
				case "block":
					rule.action = SiteRule.BLOCK;
					break;

				case "allow":
					rule.action = SiteRule.ALLOW;
					break;

				case "check":
					rule.action = SiteRule.CHECK;
					break;
			}

			rule.site = site;
			rule.closeAsync = ABPPAUtils.isChecked("delay");

			return true;
		}
		
		return false;
	},

	cancel: function()
	{
		if(!this.edit)
		{
			var rule = window.arguments[0];
			rule.clear();
		}
	},

	remove: function()
	{
		var rule = window.arguments[0];
		rule.clear();
	},

	onActionChanged: function()
	{
		var allow = ABPPA.prefs.allowTabBlocking;

		var value = ABPPAUtils.E("type").value;

		switch(value)
		{
			case "block":
				ABPPAUtils.E("regular").checked = false;
				ABPPAUtils.E("regular").disabled = true;
				ABPPAUtils.E("delay").disabled = false;
				ABPPAUtils.E("alerts").disabled = false;
				break;

			case "allow":
				ABPPAUtils.E("regular").disabled = false;
				ABPPAUtils.E("delay").disabled = true;
				ABPPAUtils.E("alerts").disabled = false;
				break;

			case "check":
				ABPPAUtils.E("regular").disabled = false;
				ABPPAUtils.E("delay").disabled = false;
				ABPPAUtils.E("alerts").checked = false;
				ABPPAUtils.E("alerts").disabled = true;
				break;
		}
	},

	onTabsClicked: function()
	{
		var allow = ABPPA.prefs.allowTabBlocking;

		if(ABPPAUtils.E("type").value == "block" && !allow && ABPPAUtils.E("tabs").checked)
		{
			params = {
				accepted: false
			};
			openDialog("chrome://adblockpopups/content/tabwarning.xul", "", "centerscreen,chrome,modal,resizable=no", params);

			if(!params.accepted)
			{
				ABPPAUtils.E("tabs").checked = false;
			}
		}
	}
};