/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var Composer =
{
	patternElement: null,
	url: null,
	win: null,
	originalUrl: null,
	openerUrl: null,
	isPopup: false,

	onLoad: function()
	{
		this.url = window.arguments[0];
		this.win = window.arguments[1];
		this.originalUrl = window.arguments[2];
		this.openerUrl = window.arguments[3];
		this.isPopup = window.arguments[4];

		this.patternElement = ABPPAUtils.E('expression');

		var uri = ABPPA.createURI(this.originalUrl);
		if(!uri || (uri.scheme !== 'http' && uri.scheme !== 'https'))
		{
			this.originalUrl = null;
		}

		if(this.originalUrl == this.url)
		{
			this.originalUrl = null;
		}

		if(!this.originalUrl)
		{
			ABPPAUtils.E('filter-source-box').hidden = true;
		}

		this.fillPatterns();

		if(!this.isPopup || !ABPIntegration.installed)
		{
			ABPPAUtils.E('blocking-type').value = 'all';
			this.updateBlockingType();

			if(!ABPIntegration.installed)
			{
				ABPPAUtils.E('blocking-type-this').disabled = true;
			}
		}
	},

	getUrl: function()
	{
		var url = this.url;

		if(this.originalUrl && ABPPAUtils.E('filter-source').value == 'original')
		{
			url = this.originalUrl;
		}

		return url;
	},

	stripSlashes: function(url)
	{
		var end = url.length - 1;

		while(end > 0 && url.charAt(end) == '/')
		{
			end--;
		}

		return url.substr(0, end + 1);
	},

	updateBlockingType: function()
	{
		var elementsToDisable = ['filter-source', 'expression', 'patterns', 'customPattern'];
		var disabled = true;
		var acceptButton = document.documentElement.getButton("accept");

		if(ABPPAUtils.E('blocking-type').value == 'this')
		{
			acceptButton.label = this.addFilterString;
			disabled = false;
		}
		else
		{
			if(!this.addFilterString)
			{
				this.addFilterString = acceptButton.label;
			}

			acceptButton.label = ABPPAUtils.getTranslation("continue");
		}

		for(var i = 0; i < elementsToDisable.length; i++)
		{
			ABPPAUtils.E(elementsToDisable[i]).disabled = disabled;
		}
	},

	updateFilterSource: function()
	{
		this.fillPatterns();
	},

	fillPatterns: function()
	{
		var url = this.getUrl();

		this.patternElement.value = url;

		var list = ABPPAUtils.E('patterns');

		while(list.itemCount > 1)
		{
			list.removeItemAt(0);
		}

		var parts = this.stripSlashes(url).split('/');

		for(var i = 3; i < parts.length; i++)
		{
			var curPattern = parts.slice(0, i).join('/') + '/';

			var node = document.createElement('radio');
			node.setAttribute('label', curPattern)
			node.setAttribute('value', curPattern);

			list.insertBefore(node, list.firstChild);
		}

		var node = document.createElement('radio');
		node.setAttribute('label', url)
		node.setAttribute('value', url);

		list.insertBefore(node, list.firstChild);
		
		if(url.charAt(url.length - 1) != '/' && 
			list.childNodes.length >= 4)
		{
			list.selectedIndex = 1;
		}
		else
		{
			list.selectedIndex = 0;
		}

		this.updatePatternSelection();

		ABPPAUtils.E('customPattern').value = url;
	},

	updatePatternSelection: function()
	{
		var list = ABPPAUtils.E('patterns');

		if(list.selectedIndex <= list.childNodes.length - 2)
		{
			this.patternElement.value = list.value;

			var type = ABPIntegration.popupsSupported() ? "$popup" : "$other";
			this.patternElement.value += type;
		}
		else
		{
			this.updateCustomPattern();
		}
	},

	updateCustomPattern: function()
	{
		this.patternElement.value = ABPPAUtils.E('customPattern')
			                          .value;

		var type = ABPIntegration.popupsSupported() ? "$popup" : "$other";
		this.patternElement.value += type;
	},

	selectCustom: function()
	{
		var list = ABPPAUtils.E('patterns');

		list.selectedIndex = list.childNodes.length - 1;
		this.updateCustomPattern();
	},

	updatePopupSetting: function()
	{
		this.updatePatternSelection();
	},

	addExpression: function()
	{
		if(ABPPAUtils.E('blocking-type').value == 'all')
		{
			var url = this.openerUrl;

			if(!url)
			{
				url = this.getUrl();
			}

			var host = '';

			if(url.match(/^https?\:\/\/(.+)$/i))
			{
				var uri = ABPPA.createURI(url);

				if(uri)
				{
					host = uri.host.replace(/^www\./, "");
				}
			}

			var rule = new SiteRule(host, SiteRule.BLOCK, SiteRule.CHECK_WINDOWS);

			openDialog("chrome://adblockpopups/content/addsite.xul", "", "centerscreen,chrome,modal,resizable=no", rule);

			if(rule.isValid)
			{
				ABPPA.addRule(rule);
				ABPPA.saveRules();

				if(this.win && this.isPopup && this.openerUrl && rule.action == SiteRule.BLOCK)
				{
					this.win.close();
				}

				return true;
			}

			return false;
		}
		if(this.patternElement.value.length)
		{
			var url = this.getUrl();
			var uri = ABPPA.createURI(url);

			ABPIntegration.addPattern(this.patternElement.value);

			if(this.win && this.isPopup && ABPIntegration.isBlacklisted(uri, uri))
			{
				this.win.close();
			}
		}

		return true;
		
	}
	
};