/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var ABPPAOverlay = 
{
	timeout: null,
	ABPMenus: ['abp-toolbar', 'abp-menuitem'],
	optionMenus: [],

	init: function()
	{
		ABPPA.registerWindow(window);
		ABPPAUtils.E('blockedPopupOptions').addEventListener('popupshown', this.onPopupMenuShown.bind(this), false);
		ABPPAUtils.E('contentAreaContextMenu').addEventListener('popupshowing', this.onContextMenuShowing.bind(this), false);
		ABPPAUtils.E('abppa-notification-popup').addEventListener('popupshowing', this.onPopupNotificationMenuShowing.bind(this), false);
		ABPPAUtils.E('abppa-notification-link').addEventListener('popupshowing', this.onLinkNotificationMenuShowing.bind(this), false);
		ABPPAUtils.E('abppa-notification-alert').addEventListener('popupshowing', this.onAlertNotificationMenuShowing.bind(this), false);
		this.createOptionsMenus();
	},

	isContextMenuDisabled: function()
	{
		if(!gBrowser.contentWindow.location.toString().match(/^https?\:\/\/(.+)$/i))
		{
			return true;
		}

		return false;
	},

	onContextMenuShowing: function(event)
	{
		var disableItems = this.isContextMenuDisabled();

		if(ABPPA.prefs.alwaysShowContextMenuItem)
		{
			ABPPAUtils.E('abppa-contextmenu').hidden = false;
			ABPPAUtils.E('abppa-block-popup').disabled = disableItems;
			ABPPAUtils.E('abppa-popup-options').disabled = disableItems;
			return;
		}

		if(!ABPPA.prefs.showContextMenu)
		{
			ABPPAUtils.E('abppa-contextmenu').hidden = true;
			return;
		}

		ABPPAUtils.E('abppa-contextmenu').hidden = false;
		ABPPAUtils.E('abppa-block-popup').disabled = disableItems;
		ABPPAUtils.E('abppa-popup-options').disabled = disableItems;

		if(disableItems)
		{
			return;
		}

		var opener = gBrowser.contentWindow.opener;
		if(!opener)
		{
			ABPPAUtils.E('abppa-block-popup').disabled = !ABPPA.prefs.contextShowAlways;
			return;
		}

		opener = opener.top;

		var isTab = (gBrowser.getBrowserIndexForDocument(opener.document) != -1);
		var uri = ABPPA.createURI(opener.document.location.toString());
		var rule = ABPPA.getRule(uri.host, isTab);

		var showItem = false;

		if(rule)
		{
			if(rule.action == SiteRule.CHECK)
			{
				showItem = true;
			}
		}
		else if(ABPPA.prefs.defaultAction == 'check')
		{ 
			if((isTab && ABPPA.prefs.enableOnTabs) || 
			   (!isTab && ABPPA.prefs.enableOnWindows))
			{
				showItem = true;
			}
		}

		ABPPAUtils.E('abppa-block-popup').disabled = !showItem;

	},

	onMenuItemCommand: function()
	{
		if(this.isContextMenuDisabled())
		{
			return;
		}

		var win = gBrowser.contentWindow;

		var origUrl = ABPPA.restoreWindowUrl(win);
		var openerUrl = ABPPA.restoreWindowOpener(win);
		var isPopup = false;

		if(origUrl && openerUrl)
		{
			isPopup = true;
		}
		else if(gBrowser.contentWindow.opener)
		{
			openerUrl = gBrowser.contentWindow.opener.top.location.href;
			isPopup = true;
		}

		window.openDialog("chrome://adblockpopups/content/composer.xul", 
			              "_blank", 
			              "chrome,centerscreen,resizable,dialog=no,dependent",
						  gBrowser.contentDocument.location.href,
			              gBrowser.contentWindow,
			              origUrl,
						  openerUrl,
						  isPopup);
	},

	onOptionsMenuItemCommand: function()
	{
		if(this.isContextMenuDisabled())
		{
			return;
		}

		var host = '';
		var url = gBrowser.contentDocument.location.href;
		var uri = ABPPA.createURI(url);
		if(uri)
		{
			host = uri.host.replace(/^www\./, "");
		}

		var rule = ABPPA.getRuleBySite(host);
		var edit = true;

		if(!rule)
		{
			rule = new SiteRule(host, SiteRule.BLOCK, SiteRule.CHECK_WINDOWS);
			edit = false;
		}

		this.openAddSiteDialog(rule, edit, edit);

		if(!edit && rule.isValid)
		{
			ABPPA.addRule(rule);
		}
		else if(edit && !rule.isValid)
		{
			ABPPA.removeRule(rule);
			rule = null;
		}
		ABPPA.saveRules();

		if(ABPPA.prefs.returnToPreferences)
		{
			this.openOptions(rule);
		}
	},

	openOptions: function(rule)
	{
		window.openDialog("chrome://adblockpopups/content/preferences.xul", 
			              "_blank", 
			              "chrome,centerscreen,resizable,dialog=no,dependent",
			              rule);
	},

	openAddSiteDialog: function(rule, edit, removeButton)
	{
		window.openDialog("chrome://adblockpopups/content/addsite.xul", "", "centerscreen,chrome,modal,resizable=no", rule, edit, edit);
	},

	onPopupMenuShown: function(event)
	{
		var nodes = event.target.childNodes;
		var started = false;

		if (!nodes)
		{
			return;
		}

		// Find all 'Show http://xxx' menu items (located after 
		// the first separator)
		for (var i = 0; i < nodes.length; i++)
		{
			if (started && nodes[i].tagName == 'menuitem' &&
				nodes[i].getAttribute('popupWindowURI'))
			{
				nodes[i].removeAttribute('oncommand');
				nodes[i].addEventListener('command', function()
				{
					ABPPA.loadInBrowser(this.getAttribute('popupWindowURI'), true, window);
				}, true);
			}

			if(nodes[i].tagName == 'menuseparator')
			{
				started = true;
			}
		}
	},

	createOptionsMenus: function()
	{
		var name = 'abppa-contextmenu-popup';
		this.createOptionsMenu(ABPPAUtils.E(name), name, 
			ABPPAUtils.getTranslation('abppaOptionsShort'),
			ABPPAUtils.getTranslation('abppaOptionsShortAccessKey'), null);
		name = 'abppa-notification-link';
		this.createOptionsMenu(ABPPAUtils.E(name), name, 
			ABPPAUtils.getTranslation('abppaOptions'),
			ABPPAUtils.getTranslation('abppaOptionsAccessKey'), ABPPAUtils.E('abppa-notification-link-disable'));
		name = 'abppa-notification-popup';
		this.createOptionsMenu(ABPPAUtils.E(name), name, 
			ABPPAUtils.getTranslation('abppaOptions'),
			ABPPAUtils.getTranslation('abppaOptionsAccessKey'), ABPPAUtils.E('abppa-notification-popup-disable'));
		name = 'abppa-notification-alert';
		this.createOptionsMenu(ABPPAUtils.E(name), name, 
			ABPPAUtils.getTranslation('abppaOptions'),
			ABPPAUtils.getTranslation('abppaOptionsAccessKey'), ABPPAUtils.E('abppa-notification-alert-disable'));

		window.setTimeout(this.injectABPMenus.bind(this, 0), 200);
	},

	injectABPMenus: function(retries)
	{
		var menus = this.ABPMenus;

		for(var a = 0; a < menus.length; a++)
		{
			if(ABPPAUtils.E('abppa-' + menus[a] + '-menu'))
			{
				// Menu has already been initialized
				continue;
			}

			var menu = ABPPAUtils.E(menus[a] + '-popup');

			if(!menu)
			{
				if(retries < 4)
				{
					window.setTimeout(this.injectABPMenus.bind(this, retries + 1), 200);
				}
				return;
			}

			var prevElement = null;

			for(var i = 0; i < menu.childNodes.length; i++)
			{
				if(menu.childNodes[i].id == menus[a] + '-options' && i < menu.childNodes.length - 1)
				{
					prevElement = menu.childNodes[i+1];
					break;
				}
			}

			this.createOptionsMenu(menu, menus[a], 
				ABPPAUtils.getTranslation('abppaOptions'), 
				ABPPAUtils.getTranslation('abppaOptionsAccessKey'),
				prevElement);
		}
	},

	createOptionsMenu: function(menu, id, title, accessKey, prevElement)
	{
		function createDomElement(aElement, aLabel, aAccessKey, aParent) {
			const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
			var item = document.createElementNS(XUL_NS, aElement);
			if(aLabel)
			{
				item.setAttribute("label", aLabel);
			}
			if(aAccessKey)
			{
				item.setAttribute("accesskey", aAccessKey);
			}
			if(aParent)
			{
				aParent.appendChild(item);
			}
			return item;
		}

		var self = this;
		var newMenu = createDomElement('menu', title, accessKey);
		newMenu.setAttribute('id', 'abppa-' + id + '-menu');
		var newMenuPopup = createDomElement('menupopup', null, null, newMenu);
		newMenuPopup.addEventListener('popupshowing', function(){self.updateOptionsMenus();});
		var menuItem1 = createDomElement('menuitem', ABPPAUtils.getTranslation('abppaEnable'), ABPPAUtils.getTranslation('abppaEnableAccessKey'), newMenuPopup);
		menuItem1.setAttribute('type', 'checkbox');
		menuItem1.setAttribute('checked', ABPPA.enabled);
		menuItem1.setAttribute('id', 'abppa-' + id + '-enable');
		menuItem1.addEventListener('command', function(){self.toggleEnabled();});
		var menuItem2 = createDomElement('menuitem', ABPPAUtils.getTranslation('abppaPreferences'), ABPPAUtils.getTranslation('abppaPreferencesAccessKey'), newMenuPopup);
		menuItem2.addEventListener('command', function(){self.openOptions();});

		if(!prevElement)
		{
			menu.appendChild(newMenu);
		}
		else
		{
			menu.insertBefore(newMenu, prevElement);
		}

		this.optionMenus.push(id);
	},

	updateOptionsMenus: function()
	{
		var menus = this.optionMenus;

		for(var a = 0; a < menus.length; a++)
		{
			var element = ABPPAUtils.E('abppa-' + menus[a] + '-enable');

			if(!element)
			{
				continue;
			}

			element.setAttribute('checked', ABPPA.enabled);
		}
	},

	toggleEnabled: function()
	{
		var p = ABPPA.ownPrefBranch;
		p.setBoolPref('enabled', !ABPPA.enabled);
		this.updateOptionsMenus();
	},

	onAllowPopupCommand: function()
	{
		this.onAllowCommand('abppa-notification-popup-allow', SiteRule.BLOCK, SiteRule.CHECK_WINDOWS | SiteRule.CHECK_TABS);
	},

	onAllowLinkCommand: function()
	{
		this.onAllowCommand('abppa-notification-link-allow', SiteRule.CHECK, SiteRule.CHECK_REGULAR);
	},

	onAllowAlertCommand: function()
	{
		this.onAllowCommand('abppa-notification-alert-allow', SiteRule.BLOCK, SiteRule.CHECK_DIALOGS);
	},

	onAllowCommand: function(name, blockAction, type)
	{
		var el = ABPPAUtils.E(name);
		var host = el.getAttribute('abppa-blocked-host');
		var action = el.getAttribute('abppa-action');
		var ruleAction = (action == 0) ? SiteRule.ALLOW : blockAction;
		var rule = new SiteRule(host, ruleAction, type, false);
		this.openAddSiteDialog(rule, false, false);
		if(rule.isValid)
		{
			ABPPA.addRule(rule);
			ABPPA.saveRules();
		}
	},

	onDisablePopupNotificationCommand: function()
	{
		this.onDisableNotificationCommand('popupWarning');
	},

	onDisableLinkNotificationCommand: function()
	{
		this.onDisableNotificationCommand('linkWarning');
	},

	onDisableAlertNotificationCommand: function()
	{
		this.onDisableNotificationCommand('alertWarning');
	},

	onDisableNotificationCommand: function(name)
	{
		ABPPA.ownPrefBranch.setIntPref(name, 0);
		gBrowser.getNotificationBox().removeCurrentNotification();
	},

	onPopupNotificationMenuShowing: function()
	{
		var strings = {
			allowString: 'popupBlockedAllow',
			allowStringAccessKey: 'popupBlockedAllowAccessKey',
			blockString: 'popupBlockedBlock',
			blockStringAccessKey: 'popupBlockedBlockAccessKey'
		};
		this.onNotificationMenuShowing('abppa-notification-popup-allow', SiteRule.CHECK_WINDOWS | SiteRule.CHECK_TABS, strings);
	},

	onLinkNotificationMenuShowing: function()
	{
		var strings = {
			allowString: 'linkBlockedAllow',
			allowStringAccessKey: 'linkBlockedAllowAccessKey',
			blockString: 'linkBlockedBlock',
			blockStringAccessKey: 'linkBlockedBlockAccessKey'
		};
		this.onNotificationMenuShowing('abppa-notification-link-allow', SiteRule.CHECK_REGULAR, strings);
	},

	onAlertNotificationMenuShowing: function()
	{
		var strings = {
			allowString: 'alertBlockedAllow',
			allowStringAccessKey: 'alertBlockedAllowAccessKey',
			blockString: 'alertBlockedBlock',
			blockStringAccessKey: 'alertBlockedBlockAccessKey'
		};
		this.onNotificationMenuShowing('abppa-notification-alert-allow', SiteRule.CHECK_DIALOGS, strings);
	},

	onNotificationMenuShowing: function(name, type, strings)
	{
		var el = ABPPAUtils.E(name);
		var strb = ABPPAUtils.E("abppa-string-bundle");
		var host = el.getAttribute('abppa-blocked-host');
		var rule = ABPPA.getRule(host, type);
		var allow = true;
		if(rule && rule.action == SiteRule.ALLOW)
		{
			allow = false;
		}
		if(allow)
		{
			el.setAttribute('label', strb.getFormattedString(strings.allowString, [host]));
			el.setAttribute('accesskey', strb.getString(strings.allowStringAccessKey));
			el.setAttribute('abppa-action', 0);
		}
		else
		{
			el.setAttribute('label', strb.getFormattedString(strings.blockString, [host]));
			el.setAttribute('accesskey', strb.getString(strings.blockStringAccessKey));
			el.setAttribute('abppa-action', 1);
		}
	}
};


window.addEventListener('load', ABPPAOverlay.init.bind(ABPPAOverlay), false);