/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var TreeView = 
{
	rowData: [],
	groupData: [],
	visibleData: [],
	treeBox: null,	
	selection: null,
	aserv: null,

	init: function()
	{
		if (Components.interfacesByID["{C06DC4D3-63A2-4422-A0A3-5F2EDDECA8C1}"])
		{
			// use legacy implementations
			this.getCellProperties = this.getCellProperties_legacy;
			this.getColumnProperties = this.getColumnProperties_legacy;
			this.getRowProperties = this.getRowProperties_legacy;
		}

		this.aserv = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService);

		this.rowData = ABPPA.rules;

		this.groupData.push({name: ABPPAUtils.getTranslation("myRules"), 
			                 data: ABPPA.rules, 
			                 open: true,
							 enabled: true});
		
		for(var i in ABPPA.subscriptions)
		{
			if(ABPPA.subscriptions[i].rules.length)
			{
				this.groupData.push({name: ABPPAUtils.getTranslation("adblockPlusSubscription") + " " + 
					                       ABPPA.subscriptions[i].title, 
					                 data: ABPPA.subscriptions[i].rules, 
					                 open: true,
					                 enabled: true});
			}
		}

		this.updateVisibleData();
	},

	updateVisibleData: function()
	{
		this.visibleData = [];

		for(var i = 0; i < this.groupData.length; i++)
		{
			this.visibleData.push([this.groupData[i].name, "",  true, true,
				                   this.groupData[i], i]); 

			var groupIndex = this.visibleData.length - 1;
			var hasEnabled = (this.groupData[i].data.length == 0);

			for(var j = 0; j < this.groupData[i].data.length; j++)
			{
				var rule = this.groupData[i].data[j];

				if(this.groupData[i].open)
				{
					this.visibleData.push([rule.site, this.actionToString(rule), 
						                       rule.enabled, false, rule, j]); 
				}

				if(rule.enabled) hasEnabled = true;
			}
				
			this.visibleData[groupIndex][2] = hasEnabled;
		}
	},
	
	get rowCount() 
	{ 
		return this.visibleData.length;
	},

	setTree: function(treeBox)
	{ 
		this.treeBox = treeBox; 
	},

	getCellText: function(idx, column) 
	{
		if(column.id == 'actioncol')
		{
			return this.visibleData[idx][1];
		}
		else if(column.id == 'sitecol')
		{
			return this.visibleData[idx][0];
		}

		return "";
	},

	actionToString: function(rule)
	{
		var str = "";

		switch(rule.action)
		{
			case SiteRule.ALLOW:
				str += ABPPAUtils.getTranslation("allowString");
				break;
			case SiteRule.BLOCK:
				str += ABPPAUtils.getTranslation("blockString");
				break;
			case SiteRule.CHECK:
				str += "Adblock Plus";
				break;
			default:
				return "";
		}

		var chars = [];

		if(rule.hasWindowType(SiteRule.CHECK_WINDOWS))
			chars.push(ABPPAUtils.getTranslation("windowsChar"));

		if(rule.hasWindowType(SiteRule.CHECK_TABS))
			chars.push(ABPPAUtils.getTranslation("tabsChar"));

		if(rule.hasWindowType(SiteRule.CHECK_REGULAR))
			chars.push(ABPPAUtils.getTranslation("regularLinksChar"));

		if(rule.hasWindowType(SiteRule.CHECK_DIALOGS))
			chars.push(ABPPAUtils.getTranslation("alertsChar"));

		if(chars.length)
		{
			str += " (";
			
			for(var i = 0; i < chars.length; i++)
			{
				if(i > 0)
				{
					str += ",";
				}

				str += chars[i];
			}

			str += ")";
		}

		return str;

	},

	getRowData: function(idx)
	{
		if(this.isContainer(idx)) return false;

		return this.visibleData[idx][4];
	},

	getCellValue: function(idx, column) 
	{
		return this.isEnabled(idx); 
	},
	
	setCellText: function(idx, column, value)
	{
		return false;
	},

	isContainer: function(idx)
	{ 
		return this.visibleData[idx][3];
	},	
	
	isContainerOpen: function(idx)
	{
		return this.visibleData[idx][4].open;
	},	

	isContainerEmpty: function(idx)
	{
		return false;
	},	
	
	isSeparator: function(idx) 
	{ 
		return false; 
	},	
	
	isSorted: function()
	{ 
		return false; 
	},	

	isEditable: function(idx, column)
	{ 
		return false;
	},	

	isEditableRow: function(idx)
	{
		return (!this.isContainer(idx) && this.getParentIndex(idx) == 0);
	},

	isEnabled: function(idx)
	{
		return this.visibleData[idx][2];
	},
	
	getParentIndex: function(idx) 
	{	
		if(this.isContainer(idx)) return -1;

		for (var t = idx - 1; t >= 0 ; t--) 
		{
			if (this.isContainer(t)) return t;
	    }
	},	

	getLevel: function(idx)
	{	
		if(this.isContainer(idx)) return 0;

		return 1;
	},	
	
	hasNextSibling: function(idx, after) 
	{
	    var thisLevel = this.getLevel(idx);
	    for (var t = after + 1; t < this.visibleData.length; t++) {
	      var nextLevel = this.getLevel(t);
	      if (nextLevel == thisLevel) return true;
	      if (nextLevel < thisLevel) break;
	    }
	    return false;
	},

	toggleOpenState: function(idx) 
	{	
		if(!this.isContainer(idx)) return;
		
		var itemCount = this.visibleData[idx][4].data.length;

		if(this.isContainerOpen(idx))
		{
			this.visibleData[idx][4].open = false;
			this.updateVisibleData();
			this.treeBox.rowCountChanged(idx + 1, -itemCount);
		}
		else
		{
			this.visibleData[idx][4].open = true;
			this.updateVisibleData();
			this.treeBox.rowCountChanged(idx + 1, itemCount);
		}

		this.treeBox.invalidateRow(idx);

	},

	toggleDisabled: function(idx, value)
	{
		if(typeof value == "undefined")
		{
			value = !this.isEnabled(idx);
		}

		if(this.isContainer(idx))
		{
			for(var i = 0; i < this.visibleData[idx][4].data.length; i++)
			{
				this.visibleData[idx][4].data[i].enabled = value;

				if(this.isContainerOpen(idx))
				{
					this.visibleData[idx + i + 1][2] = value;
					this.treeBox.invalidateRow(idx + i + 1);
				}
			}
		}
		else
		{
			this.visibleData[idx][4].enabled = value;
		}

		this.visibleData[idx][2] = value;
		this.treeBox.invalidateRow(idx);

	},

	removeRow: function(idx)
	{
		if(this.isContainer(idx)) return false;
		else if(!this.isEditableRow(idx)) return false;

		var parentIndex = this.getParentIndex(idx);
		var ruleIndex = this.visibleData[idx][5];

		this.visibleData[parentIndex][4].data.splice(ruleIndex, 1);
		this.updateVisibleData();
		this.treeBox.rowCountChanged(idx, -1);
		return true;
	},

	appendRow: function(data)
	{
		if(!this.isContainerOpen(0))
		{
			this.toggleOpenState(0);
		}

		var i = 1;

		while(i < this.rowCount && !this.isContainer(i))
		{
			if(this.visibleData[i][4].overLaps(data))
			{
				this.removeRow(i);
				i--;
			}

			i++;
		}

		this.groupData[0].data.push(data);
		this.updateVisibleData();
		var last = this.groupData[0].data.length;
		this.treeBox.rowCountChanged(last, 1);
		this.treeBox.invalidateRow(last - 1);
		this.selection.select(last);
	},

	changeRow: function(idx, value)
	{
		if(!this.isEditableRow(idx)) return false;

		var parentIndex = this.getParentIndex(idx);
		var ruleIndex = this.visibleData[idx][5];

		this.visibleData[parentIndex][4].data[ruleIndex] = value;
		this.updateVisibleData();
		this.treeBox.invalidateRow(idx);
		this.selection.select(idx);

		return true;
	},

	saveTreeData: function()
	{
		ABPPA.saveRules();
	},

	selectRule: function(rule)
	{
		for(var i = 0; i < this.visibleData.length; i++)
		{
			if(this.visibleData[i].length > 4 && this.visibleData[i][4].toString() == rule.toString())
			{
				this.selection.select(i);
				return true;
			}
		}
		return false;
	},

	sortFunc: function(a, b, column, order)
	{
		switch(column)
		{
			case 'sitecol':
				if(a.site < b.site) return -1 * order;
				if(a.site > b.site) return 1 * order;
				break;

			case 'actioncol':
				var aVal = this.actionToString(a);
				var bVal = this.actionToString(b);
				if(aVal < bVal) return -1 * order;
				if(aVal > bVal) return 1 * order;
				break;

			case 'enabledcol':
				if(!a.enabled && b.enabled) return -1 * order;
				if(a.enabled && !b.enabled) return 1 * order;
				break;
			}

		return 0;	
	},

	sort: function(column, order)
	{
		var self = this;

		for(var i = 0; i < this.groupData.length; i++)
		{
			this.groupData[i].data.sort(function(a, b) 
			{
				return self.sortFunc(a, b, column, order);
			});
		}

		this.updateVisibleData();
		this.treeBox.invalidate();
	},
	
	setCellValue: function(idx, column, value) {},
	getImageSrc: function(idx, column) {},	
	getProgressMode : function(idx,column) {},	
	cycleHeader: function(col, elem) {},	
	selectionChanged: function() {},	
	cycleCell: function(idx, column) {},	
	performAction: function(action) {},	
	performActionOnCell: function(action, index, column) {},
	getRowProperties: function(row)
	{
		var str = "";
		str += this.aserv.getAtom("selected-" + this.selection.isSelected(row));
		str += " " + this.aserv.getAtom("enabled-" + this.isEnabled(row));
		return str;
	},
	getCellProperties: function(row, column) { return ""; },
	getColumnProperties: function(column) { return ""; },
	
	getRowProperties_legacy: function(row, props)
	{
		props.AppendElement(this.aserv.getAtom("selected-" + this.selection.isSelected(row)));
		props.AppendElement(this.aserv.getAtom("enabled-" + this.isEnabled(row)));
	},	
	getCellProperties_legacy: function(idx, column, props) {},	
	getColumnProperties_legacy: function(column, props) {},	
};	

var Preferences = 
{
	onLoad: function()
	{
		if(ABPIntegration.installed || ABPPA.prefs.disableAbpWarning)
		{
			ABPPAUtils.E('abpwarning').hidden = true;
		}
		else
		{
			ABPPAUtils.E('abpwarning').hidden = false;
		}
		ABPPAUtils.E('adblockpopups_toggle').setAttribute("checked", ABPPA.prefs.enabled);
		ABPPAUtils.E('enable_newtabs').setAttribute("checked", ABPPA.prefs.enableOnTabs);
		ABPPAUtils.E('enable_newwindows').setAttribute("checked", ABPPA.prefs.enableOnWindows);
		ABPPAUtils.E('enable_links').setAttribute("checked", ABPPA.prefs.enableOnLinks);
		ABPPAUtils.E('enable_alerts').setAttribute("checked", ABPPA.prefs.enableOnAlerts);
		ABPPAUtils.E('show_context_menu').setAttribute("checked", ABPPA.prefs.showContextMenu);
		ABPPAUtils.E('enable_notification').setAttribute("checked", !ABPPA.prefs.disableBrowserWarning);
		ABPPAUtils.E('default-action').value = ABPPA.prefs.defaultAction;
		TreeView.init();
		ABPPAUtils.E('sites').view = TreeView;
		ABPPAUtils.E("listStack").addEventListener("keypress", this.onListKeyPress, true);
		this.sortRules();
		var rule = null;
		if(window.arguments && window.arguments.length > 0)
		{
			rule = window.arguments[0];
		}
		setTimeout(function(){
			if(!rule || !TreeView.selectRule(rule))
			{
				TreeView.selection.select(0);
			}
		}, 0);
	},

	findRDFID: function(extDB, sID)
	{
		var prefixes = [
			"urn:mozilla:extension:", 	// Firefox 1.0.x
			"urn:mozilla:item:"			// Firefox 1.5
		];
		var rdfs = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
		
		for (var i in prefixes)
		{
			var sRDFID = prefixes[i] + sID;
			if (extDB.GetTarget(rdfs.GetResource(sRDFID), 
								rdfs.GetResource("http://www.mozilla.org/2004/em-rdf#name"), 
								true) !== null)
				return sRDFID;
		}
		
		return null;
	},

	openAbout: function()
	{
		var sID = "adblockpopups@jessehakanen.net";
		try {
			Components.utils["import"]("resource://gre/modules/AddonManager.jsm");
			AddonManager.getAddonByID(sID, function(addon) {
				
				if (addon)
					openDialog("chrome://mozapps/content/extensions/about.xul", "", "chrome,centerscreen,modal", addon);
			});
		} catch (ex) {
			var extMgr = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
			var extDB = extMgr.datasource;
			var sRDFID = this.findRDFID(extDB, sID);
			if (sRDFID !== null)
				openDialog("chrome://mozapps/content/extensions/about.xul", "", "chrome,centerscreen,modal", sRDFID, extDB);
		}
	},

	save: function()
	{
		var p = ABPPA.ownPrefBranch;

		p.setBoolPref('enabled', ABPPAUtils.isChecked('adblockpopups_toggle'));
		p.setBoolPref('enableOnTabs', ABPPAUtils.isChecked('enable_newtabs'));
		p.setBoolPref('enableOnWindows', ABPPAUtils.isChecked('enable_newwindows'));
		p.setBoolPref('enableOnLinks', ABPPAUtils.isChecked('enable_links'));
		p.setBoolPref('enableOnAlerts', ABPPAUtils.isChecked('enable_alerts'));
		p.setBoolPref('showContextMenu', ABPPAUtils.isChecked('show_context_menu'));
		p.setCharPref('defaultAction', ABPPAUtils.E('default-action').value);
		p.setBoolPref('disableBrowserWarning', !ABPPAUtils.isChecked('enable_notification'));

		TreeView.saveTreeData();
	},

	addSite: function()
	{
		var rule = new SiteRule;

		openDialog("chrome://adblockpopups/content/addsite.xul", "", "centerscreen,chrome,modal,resizable=no", rule);

		if(!rule.isValid)
		{
			return;
		}

		TreeView.appendRow(rule);
		TreeView.selectRule(rule);

	},

	editSite: function(row)
	{
		if(TreeView.isContainer(row) || !TreeView.isEditableRow(row))
		{
			return;
		}

		var rule = TreeView.getRowData(row);

		openDialog("chrome://adblockpopups/content/addsite.xul", "", "centerscreen,chrome,modal,resizable=no", rule, true);

		if(!rule.isValid)
		{
			return;
		}

		TreeView.changeRow(row, rule);

		return;

	},

	onListClick: function(e)
	{
		if (e.button != 0)
			return;

		let row = {};
		let col = {};
		TreeView.treeBox.getCellAt(e.clientX, e.clientY, row, col, {});

		if (!col.value || col.value.id != "enabledcol")
			return;

		TreeView.toggleDisabled(row.value);
	},

	onListDoubleClick: function(e)
	{
		if (e.button != 0)
			return;

		let row = {};
		let col = {};
		TreeView.treeBox.getCellAt(e.clientX, e.clientY, row, col, {});

		if (!col.value || col.value.id == "enabledcol")
			return;

		this.editSite(row.value);
	},

	onListKeyPress: function(e)
	{
		if(ABPPAUtils.E('sites').editingRow != -1)
		{
			return;
		}

		if (e.keyCode == e.DOM_VK_DELETE || e.keyCode == e.DOM_VK_BACK_SPACE)
		{
			Preferences.removeSelected();
		}
		else if(e.keyCode == e.DOM_VK_RETURN || e.keyCode == e.DOM_VK_ENTER)
		{
			e.preventDefault();
			Preferences.editSelected();
		}
	},

	toggleDisabled: function(value)
	{
		var numRanges = TreeView.selection.getRangeCount();
		var start = {};
		var end = {};

		for(var t = 0; t < numRanges; t++)
		{
			TreeView.selection.getRangeAt(t, start, end);	

			for(var v = start.value; v <= end.value; v++)
			{
				TreeView.toggleDisabled(v, value);
			}
		}

	},

	editSelected: function()
	{
		var numRanges = TreeView.selection.getRangeCount();
		var start = {};
		var end = {};

		TreeView.selection.getRangeAt(0, start, end);

		this.editSite(start.value);
	},

	removeSelected: function()
	{
		var numRanges = TreeView.selection.getRangeCount();
		var start = {};
		var end = {};
		var deleted = false;

		for(var t = 0; t < numRanges; t++)
		{
			TreeView.selection.getRangeAt(0, start, end);

			var removed = 0;

			for(var v = start.value; v <= end.value; v++)
			{
				if(TreeView.removeRow(v - removed))
				{
					deleted = true;
					removed++;
				}
			}
		}

		if(deleted)
		{
			TreeView.selection.clearSelection();
		}
	},

	sortRules: function(column)
	{
		var columnName;
		var tree = ABPPAUtils.E("sites");
		var order = tree.getAttribute("sortDirection") == "ascending" ? 1 : -1;

		if(column)
		{
			columnName = column.id;

			if(tree.getAttribute("sortResource") == columnName)
			{
				order *= -1;
			}
		}
		else
		{
			columnName = tree.getAttribute("sortResource");
		}

		TreeView.sort(columnName, order);

		tree.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
		tree.setAttribute("sortResource", columnName);

		var cols = tree.getElementsByTagName("treecol");

		for (var i = 0; i < cols.length; i++)
		{
			cols[i].removeAttribute("sortDirection");
		}

		ABPPAUtils.E(columnName).setAttribute("sortDirection", order == 1 ? "ascending" : "descending");

	},

	setHotkey: function()
	{
		openDialog("chrome://adblockpopups/content/hotkey.xul", "", "centerscreen,chrome,modal,resizable=no");
	},

	exportRules: function()
	{
		var picker = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
		picker.init(window, ABPPAUtils.getTranslation("exportRules"), picker.modeSave);
		picker.defaultExtension = ".txt";
		picker.appendFilters(picker.filterText);
		picker.appendFilters(picker.filterAll);

		if (picker.show() != picker.returnCancel)
		{
			var lines = ["[Adblock Plus Pop-up Addon]"];

			for(var i = 0; i < ABPPA.rules.length; i++)
			{
				lines.push(ABPPA.rules[i].toString());
			}

			try
			{
				var fileStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
				fileStream.init(picker.file, 0x02 | 0x08 | 0x20, 0644, 0);

				var stream = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
				stream.init(fileStream, "UTF-8", 16384, Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);

				stream.writeString(lines.join("\r\n"));
		
				stream.close();
			}
			catch (e)
			{
				alert(ABPPAUtils.getTranslation("unableToWrite"));
			}
		}
	},

	importRules: function()
	{
		var picker = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
		picker.init(window, ABPPAUtils.getTranslation("importRules"), picker.modeOpen);
		picker.defaultExtension = ".txt";
		picker.appendFilters(picker.filterText);
		picker.appendFilters(picker.filterAll);

		if(picker.show() != picker.returnCancel)
		{
			var fileStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
			fileStream.init(picker.file, 0x01, 0444, 0);

			var stream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance(Components.interfaces.nsIConverterInputStream);
			stream.init(fileStream, "UTF-8", 16384, Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
			stream = stream.QueryInterface(Components.interfaces.nsIUnicharLineInputStream);

			var lines = [];
			var line = {value: null};
			while(stream.readLine(line))
			{
				lines.push(line.value);
			}
			if(line.value)
			{
				lines.push(line.value);
			}

			stream.close();

			for(var i = 0; i < lines.length; i++)
			{
				// Trim possible white spaces
				lines[i] = lines[i].replace(/^\s+|\s+$/g, '');

				if(lines[i].charAt(0) == '[')
				{
					continue;
				}

				var rule = new SiteRule;

				if(rule.fromString(lines[i]) && rule.isValid)
				{
					TreeView.appendRow(rule);
				}

			}

			TreeView.saveTreeData();

		}
	},

	updateTooltip: function()
	{
		ABPPAUtils.E('action-tooltip').label = ABPPAUtils.E('default-action').selectedItem.label;
	},

	fillContext: function()
	{
		var editable = TreeView.isEditableRow(TreeView.selection.currentIndex);

		if(TreeView.selection.count == 1)
		{
			if(TreeView.isEnabled(TreeView.selection.currentIndex))
			{
				ABPPAUtils.E('context-enable').hidden = true;
				ABPPAUtils.E('context-disable').hidden = false;
			}
			else
			{
				ABPPAUtils.E('context-enable').hidden = false;
				ABPPAUtils.E('context-disable').hidden = true;
			}

			ABPPAUtils.E('context-edit').disabled = !editable;
		}
		else
		{
			ABPPAUtils.E('context-enable').hidden = false;
			ABPPAUtils.E('context-disable').hidden = false;
			ABPPAUtils.E('context-edit').disabled = true;
		}

		ABPPAUtils.E('context-remove').disabled = !editable;

		return true;
	}

}