/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ["ABPPA"];  

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;
const Cr = Components.results;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");

var myURL = "resource://adblockpopups/";
Cu.import(myURL + "RuleClasses.jsm");
Cu.import(myURL + "RedirectionCache.jsm");
Cu.import(myURL + "ABPIntegration.jsm");

const ABPPA = 
{
	addonId: "adblockpopups@jessehakanen.net",
	_enabled: true,
	loaded: false,
	maxRedirects: 0,
	disableOpen: true,
	previousRequest: null,
	checkFirstRun: true,
	rules: [],
	httpRules: [],
	windows: [],
	subscriptions: {},

	suggestedSubscriptions: {},

	prefs: { // Own preferences
		'popupAllowTime': 0,
		'requestTimeout': 0,
		'enabled': true,
		'enableOnTabs': true,
		'enableOnWindows': true,
		'enableOnLinks': false,
		'enableOnAlerts': false,
		'resolveTabs': true,
		'resolveWindows': true,
		'requestMethod': 0,
		'redirectionCacheSize': 0,
		'showContextMenu': true,
		'allowTabBlocking': false,
		'disableBrowserWarning': false,
		'popupWarning': 0,
		'linkWarning': 0,
		'alertWarning': 0,
		'rules' : '',
		'defaultAction': '',
		'hotkeyEnabled': false,
		'hotkey': '',
		'hotkeyPrefsEnabled': false,
		'hotkeyPrefs': '',
		'closeAsync': false,
		'closeAsyncTime': 0,
		'disableAbpWarning': false,
		'alwaysShowContextMenuItem': false,
		'returnToPreferences': true,
		'disableOpener': false
	},

	get enabled()
	{
		return (this._enabled && this.prefs.enabled);
	},

	set enabled(value)
	{
		this._enabled = value;
	},

	init: function()
	{
		if (this.loaded) return;

		this.loaded = true;

		this.wm = new WeakMap();

		this.sessionStore = Cc["@mozilla.org/browser/sessionstore;1"]
			                  .getService(Ci.nsISessionStore);
		this.popupManager = Cc["@mozilla.org/PopupWindowManager;1"]
			                  .getService(Ci.nsIPopupWindowManager);
  		this.ioService = Cc["@mozilla.org/network/io-service;1"]
			                  .getService(Ci.nsIIOService);
		var prefService = Cc["@mozilla.org/preferences-service;1"]
			                  .getService(Ci.nsIPrefService);
		this.threadManager = Cc["@mozilla.org/thread-manager;1"]
			                  .getService(Ci.nsIThreadManager);

		this.prefBranch = prefService.getBranch(null)
			                 .QueryInterface(Ci.nsIPrefBranch);

		this.ownPrefBranch = prefService.getBranch('extensions.adblockpluspopupaddon.')
			                 .QueryInterface(Ci.nsIPrefBranch);


		for (var i in this.prefs)
		{
			this.loadOwnPreferences(i);
		}

		this.ownPrefBranch.addObserver('', this.ownPrefObserver, false);

		var preferencesToListen = [
		    'network.http.redirection-limit',
			'dom.disable_open_during_load'
		];
	
		for (var i = 0; i < preferencesToListen.length; i++)
		{
			this.prefBranch.addObserver(preferencesToListen[i], 
			                            this.prefObserver, false);
			this.loadPreferences(preferencesToListen[i]);
		}

		this.loadRules();

		var registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
		registrar.registerFactory(this.contentPolicy.classID, this.contentPolicy.classDescription, 
		                          this.contentPolicy.contractID, this.contentPolicy);

		var catMan = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
		for each (let category in this.contentPolicy.xpcom_categories)
			catMan.addCategoryEntry(category, this.contentPolicy.classDescription, 
			                        this.contentPolicy.contractID, false, true);

		var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
		observerService.addObserver(this.httpObserver, "http-on-modify-request", false);
		observerService.addObserver(this.httpObserver, "common-dialog-loaded", false);
		observerService.addObserver(this.httpObserver, "tabmodal-dialog-loaded", false);

		this.loadSubscriptions();

		var self = this;

		ABPIntegration.addSubscriptionObserver(function(subscription, action){
			self.handleSubscriptionChange(subscription, action);
		});
	},

	loadSubscriptions: function()
	{
		var subscriptions = ABPIntegration.getSubscriptions();

		for(var i = 0; i < subscriptions.length; i++)
		{
			this.handleSubscriptionChange(subscriptions[i], 'add');

			if(this.suggestedSubscriptions[subscriptions[i].url])
			{
				this.checkFirstRun = false;
			}
		}
	},

	handleSubscriptionChange: function(subscription, action)
	{
		var title = subscription.title;

		if(this.subscriptions[title])
		{
			this.subscriptions[title].clear();

			if(action == 'remove')
			{
				return;
			}
		}
		else
		{
			if(action == 'update')
			{
				return;
			}

			this.subscriptions[title] = new RuleSubscription(title);
		}

		if (subscription.filters.length)
		{
			for(var i = 0; i < subscription.filters.length; i++)
			{
				if(RuleSubscription.isRuleString(subscription.filters[i].text))
				{
					this.subscriptions[title].addRuleFromFilter(subscription.filters[i]);
				}
			}
		}
	},

	registerWindow: function(win)
	{
		var self = this;

		win.addEventListener("unload", this.removeWindow.bind(this), false);
		this.windows.push(win);
		this.initWindowHotkey(win);

		if(this.checkFirstRun)
		{
			this.checkFirstRun = false;
			this.handleFirstRun();
		}
	},

	removeWindow: function(win)
	{
		for(var i = 0; i < this.windows.length; i++)
		{
			if(this.windows[i] == win)
			{
				this.windows.splice(i, 1);
				return;
			}
		}
	},

	closeWindow: function(win, rule)
	{
		if(rule)
		{
			if(rule.closeAsync)
			{
				if(this.prefs.closeAsyncTime > 0)
				{
					win.setTimeout(function(){
						ABPPA.runAsync(win.close, win);
					}, this.prefs.closeAsyncTime);
				}
				else
				{
					this.runAsync(win.close, win);
				}
			}
			else
			{
				win.close();
			}
		}
		else
		{
			if(this.prefs.closeAsync)
			{
				if(this.prefs.closeAsyncTime > 0)
				{
					win.setTimeout(function(){
						ABPPA.runAsync(win.close, win);
					}, this.prefs.closeAsyncTime);
				}
				else
				{
					this.runAsync(win.close, win);
				}
			}
			else
			{
				win.close();
			}
		}
	},

	handleFirstRun: function()
	{
		// Do nothing at the moment
	},

	dump: function(str)
	{
		if(!this.windows.length) return;
		this.windows[0].dump(str + "\n");
	},

	loadInBrowser: function(url, openNewTab, win)
	{
		if(!win)
		{
			if(!this.windows.length) return false;
			win = this.windows[0];
		}
		if(openNewTab)
		{
			win.gBrowser.loadOneTab(url, {inBackground: false});
		}
		else
		{
			win.gBrowser.loadURI(url);
		}
		return true;
	},

	setData: function(object, key, value)
	{
		var data = {};
		if(this.wm.has(object))
		{
			data = this.wm.get(object);
		}
		data[key] = value;
		this.wm.set(object, data);
	},

	getData: function(object, key)
	{
		var data = this.wm.get(object, null);
		if(data && typeof(data[key]) != 'undefined')
		{
			return data[key];
		}
		return null;
	},

	// Borrowed from Adblock Plus
	// Original author: Wladimir Palant
	runAsync: function(callback, thisPtr)
	{
		let params = Array.prototype.slice.call(arguments, 2);
		let runnable = {
			run: function()
			{
				callback.apply(thisPtr, params);
			}
		};
		this.threadManager.currentThread.dispatch(runnable, Ci.nsIEventTarget.DISPATCH_NORMAL);
	},

	initHotkey: function()
	{
		for(var i = 0; i < this.windows.length; i++)
		{
			this.initWindowHotkey(this.windows[i]);
		}
	},

	initWindowHotkey: function(win)
	{
		this.initHotkeySingle(win, 'hotkey', 'key-abppa-block', 
			                  'abppa-block-popup', this.prefs.hotkeyEnabled);

		this.initHotkeySingle(win, 'hotkeyPrefs', 'key-abppa-preferences',
			                  'abppa-show-preferences', this.prefs.hotkeyPrefsEnabled);
	},

	initHotkeySingle: function(win, prefname, keyid, menuid, enabled)
	{
		var key = win.document.getElementById(keyid);
		var menuItem = win.document.getElementById(menuid);

		if(!key || !menuItem) return;

		if(enabled)
		{
			var hotkey = this.prefs[prefname].toLowerCase();

			var parts = hotkey.split(' ');
			var modifiers = 'accel shift';
			var keycode = hotkey;

			if(parts.length > 1)
			{
				modifiers = parts.splice(0, parts.length - 1).join(' ');
				keycode = parts[parts.length - 1];
			}

			key.setAttribute('disabled', false);
			key.setAttribute('key', keycode);
			key.setAttribute('modifiers', modifiers);

			if(!menuItem.hasAttribute('key'))
			{
				menuItem.setAttribute('key', keyid);
			}
		}
		else
		{
			menuItem.removeAttribute('key');
			key.setAttribute('disabled', true);
		}
	},

	addRule: function(rule)
	{
		for(var i = this.rules.length - 1; i >= 0; i--)
		{
			if(this.rules[i].overLaps(rule))
			{
				this.rules.splice(i, 1);
			}
		}

		this.rules.push(rule);
	},

	removeRule: function(rule)
	{
		for(var i = this.rules.length - 1; i >= 0; i--)
		{
			if(this.rules[i] == rule)
			{
				this.rules.splice(i, 1);
			}
		}
	},

	getRuleBySite: function(site)
	{
		for(var i = 0; i < this.rules.length; i++)
		{
			if(this.rules[i].matches(site))
			{
				return this.rules[i];
			}
		}
		return null;
	},

	isWhitelisted: function(uri, win)
	{
		if(ABPIntegration.isWhitelisted(uri))
		{
			return true;
		}

		// Check if window uri is white listed in Firefox's own popup manager
		var principal = win.document.nodePrincipal;

		if (principal && this.disableOpen && this.popupManager.testPermission(principal) == this.popupManager.ALLOW_POPUP)
		{
			return true;
		}

		return false;
	},

	getTabForWindow: function(win)
	{
		var gbr = this.getDOMWindow(win).gBrowser;
		var index = gbr.getBrowserIndexForDocument(win.document);
		return gbr.tabContainer.childNodes[index];
	},

	storeWindowData: function(win, url, opener)
	{
		var tab = this.getTabForWindow(win);
		this.sessionStore.setTabValue(tab, "abppaOriginalUrl", url);
		this.sessionStore.setTabValue(tab, "abppaOpenerUrl", opener);
	},

	restoreWindowUrl: function(win)
	{
		var tab = this.getTabForWindow(win);
		return this.sessionStore.getTabValue(tab, "abppaOriginalUrl");
	},

	restoreWindowOpener: function(win)
	{
		var tab = this.getTabForWindow(win);
		return this.sessionStore.getTabValue(tab, "abppaOpenerUrl");
	},

	onDocumentClicked: function(win)
	{
		if(win.opener)
		{
			// When user clicks on a document, disable further pop-up blocking for that window
			var tab = this.getTabForWindow(win);
			this.sessionStore.setTabValue(tab, "abppaDisable", "1");;
		}
	},

	isPopup: function(win)
	{
		if(win.opener)
		{
			var tab = this.getTabForWindow(win);
			var disabled = this.sessionStore.getTabValue(tab, "abppaDisable");
			if(!disabled || disabled !== "1")
			{
				return true;
			}
		}
		return false;
	},
	
	loadPreferences: function(name)
	{
		switch (name)
		{
			case 'network.http.redirection-limit':
				this.maxRedirects = this.prefBranch.getIntPref(name);
				break;

			case 'dom.disable_open_during_load':
				this.disableOpen = this.prefBranch.getBoolPref(name);
				break;

		}

	},

	loadRules: function()
	{
		this.rules.length = 0;

		if(this.prefs.rules.length == 0)
		{
			return;
		}

		var list = this.prefs.rules.split('|');
		
		for(var i = 0; i < list.length; i++)
		{
			var rule = new SiteRule;

			if(rule.fromString(list[i]))
			{
				this.rules.push(rule);		
			}
		}

		this.reloadHttpRules();

	},

	// Store HTTP rules in an own array to speed up things
	reloadHttpRules: function()
	{
		this.httpRules.length = 0;

		for(var i = 0; i < this.rules.length; i++)
		{
			if(this.rules[i].hasWindowType(SiteRule.CHECK_REGULAR))
			{
				this.httpRules.push(this.rules[i]);
			}
		}

		for(var title in this.subscriptions)
		{
			for(var j = 0; j < this.subscriptions[title].rules.length; j++)
			{
				var r = this.subscriptions[title].rules[j];

				if(r.hasWindowType(SiteRule.CHECK_REGULAR))
				{
					this.httpRules.push(r);
				}
			}
		}
	},

	getRule: function(host, type)
	{
		var rule = null;

		for(var i = 0; i < this.rules.length; i++)
		{
			if(this.rules[i].matches(host) && this.rules[i].hasWindowType(type))
			{
				switch(this.rules[i].action)
				{
					case SiteRule.ALLOW:
						rule = this.rules[i]; // Allow rule overrides any previous rule
						break;

					case SiteRule.BLOCK:
						if(rule == null)
						{
							rule = this.rules[i]; // Block rule does not override any rule
						}
						break;

					case SiteRule.CHECK:
						if(rule == null || rule.action == SiteRule.BLOCK) // Check rule overrides only block rule
						{
							rule = this.rules[i];
						}
						break;
				}
			}
		}

		if(rule != null)
		{
			return rule;
		}

		for(var title in this.subscriptions)
		{
			for(var j = 0; j < this.subscriptions[title].rules.length; j++)
			{
				var r = this.subscriptions[title].rules[j];

				if(r.matches(host) && r.hasWindowType(type))
				{
					switch(r.action)
					{
						case SiteRule.ALLOW:
							rule = r; // Allow rule overrides any previous rule
							break;

						case SiteRule.BLOCK:
							if(rule == null)
							{
								rule = r; // Block rule does not override any rule
							}
							break;

						case SiteRule.CHECK:
							if(rule == null || rule.action == SiteRule.BLOCK) // Check rule overrides only block rule
							{
								rule = r;
							}
							break;
					}
				}
			}
		}

		if(rule != null)
		{
			return rule;
		}

		return false;
	},

	saveRules: function()
	{
		this.ownPrefBranch.setCharPref('rules', this.rulesToString());
		this.reloadHttpRules();
	},

	rulesToString: function()
	{
		var str = "";

		for(var i = 0; i < this.rules.length; i++)
		{
			str += this.rules[i].toString() + "|";
		}

		if(str.length)
		{
			str = str.substring(0, str.length - 1);
		}

		return str;
	},

	loadOwnPreferences: function(name)
	{
		var type = typeof this.prefs[name];

		if (type == 'boolean')
		{
			this.prefs[name] = this.ownPrefBranch.getBoolPref(name);
		}
		else if (type == 'number')
		{
			this.prefs[name] = this.ownPrefBranch.getIntPref(name);

			if(name == 'redirectionCacheSize')
			{
				RedirectionCache.maxSize = this.prefs[name];
			}
		}
		else if (type == 'string')
		{
			this.prefs[name] = this.ownPrefBranch.getCharPref(name);

			if(name == 'rules')
			{
				this.loadRules();
			}
		}

		if(name == 'hotkeyEnabled' || name == 'hotkey')
		{
			this.initHotkey();
		}
	},

	createURI: function(url)
	{
		var uri = false;

		try {
			uri = this.ioService.newURI(url, null, null);
		} catch (e) {
			// Invalid uri
			uri = false;
		}

		return uri;
	},

	getRealURI: function(uri, opener, win, resolve)
	{
		var target = null;

		if(target = RedirectionCache.get(uri.spec))
		{
			return this.createURI(target);	
		}

		if(!resolve)
		{
			return uri;
		}

		var channel = this.ioService.newChannelFromURI(uri)
			               .QueryInterface(Ci.nsIHttpChannel);

		channel.requestMethod = (this.prefs.requestMethod == 0) ? 'HEAD' : 'GET';

		if(opener)
		{
			channel.setRequestHeader('Referer', opener.spec, false);
		}
		
		if (this.maxRedirects > 0)
		{
			channel.redirectionLimit = this.maxRedirects;
		}

		// Follow redirects to get real uri
		var listener = new RedirectListener(uri);

		channel.notificationCallbacks = listener;

		var timedOut = false;

		if (this.prefs.requestTimeout > 0)
		{
			// Set timeout for request
			win.setTimeout(function()
			{
				if(!listener.ready)
				{
					channel.cancel(Cr.NS_BINDING_ABORTED);
					timedOut = true;
				}

			}, this.prefs.requestTimeout);

		}

		channel.open();

		if(!timedOut)
		{
			RedirectionCache.insert(uri.spec, listener.currentURI.spec);
		}

		return listener.currentURI;
	},

	// Check regular link against our lists and possibly Adblock's lists
	checkHttpURI: function(win, uri, opener)
	{
		var check = false;

		if(this.prefs.enableOnLinks)
		{
			check = true;
		}

		for(var i = 0; i < this.httpRules.length; i++)
		{
			if(this.httpRules[i].matches(opener.host))
			{
				var rule = this.httpRules[i];

				if(rule.action == SiteRule.ALLOW ||
				   rule.action == SiteRule.BLOCK) // Should not be possible
				{
					return true;
				}

				check = true;
				break;
			}
		}

		if(check)
		{
			if(this.isWhitelisted(opener, win.top))
			{
				return true;
			}

			if(ABPIntegration.isBlacklisted(uri, opener))
			{
				return false;
			}
		}

		return true;
	},

	// Checks URI against our lists and Adblock's lists
	checkURI: function(uri, win, tab, resolve, out)
	{
		var openerURI = this.createURI(win.opener.top.document.location);
		var type = tab ? SiteRule.CHECK_TABS : SiteRule.CHECK_WINDOWS;
		var openerRule = this.getRule(openerURI.host, type);
		var oldUrl = this.restoreWindowUrl(win);

		var check = true;
		out.rule = openerRule;

		if(!this.prefs.enableOnTabs && tab)
		{
			check = false;
		}
		else if(!this.prefs.enableOnWindows && !tab)
		{
			check = false;
		}

		if(openerRule)
		{
			switch(openerRule.action)
			{
				case SiteRule.ALLOW:
					this.handleWindowAllow(win);
					return true;

				case SiteRule.BLOCK:
					if(resolve)
					{
						return false;
					}
					break;

				case SiteRule.CHECK:
					check = true;
					break;
			}
		}
		else if(this.prefs.defaultAction == "allow")
		{
			this.handleWindowAllow(win);
			return true;
		}
		else if(this.prefs.defaultAction == "block")
		{
			if(!tab)
			{
				return false;
			}
			
			check = this.prefs.enableOnTabs;
		}

		if(!check)
		{
			return true;
		}

		if(this.isWhitelisted(openerURI, win.opener.top))
		{
			this.handleWindowAllow(win);
			return true;
		}

		if(ABPIntegration.isBlacklisted(uri, openerURI))
		{
			if(oldUrl)
			{
				RedirectionCache.insert(oldUrl, uri.spec);
			}

			return false;
		}

		var doResolve = resolve;

		if(doResolve)
		{
			if(tab && !this.prefs.resolveTabs)
			{
				doResolve = false;
			}
			else if(!tab && !this.prefs.resolveWindows)
			{
				doResolve = false;
			}
		}
		
		var realURI = this.getRealURI(uri, openerURI, win, doResolve);

		if(oldUrl)
		{
			RedirectionCache.insert(oldUrl, realURI.spec);
		}

		if(realURI.spec != uri.spec && ABPIntegration.isBlacklisted(realURI, openerURI))
		{
			return false;
		}

		return true;
	},

	handleWindowAllow: function(win)
	{
		if (this.prefs.disableOpener && ABPIntegration.enabled)
		{
			var originalOpener = win.opener;
			win.opener = null; // Make sure Adblock Plus doesn't block this after we have just allowed the popup
			var self = this;
			win.setTimeout(function(){
				self.runAsync(function(){ win.opener = originalOpener; }, self);
			}, 0);
		}
	},

	showPopupBlockedNotification: function(win, uri)
	{		
		if(this.prefs.popupWarning == 0)
		{
			return;
		}
		else if((this.prefs.popupWarning == 2) && this.prefs.disableBrowserWarning)
		{
			return;
		}

		var strings = 
		{
			titleStringSingle: 'popupBlocked',
			titleStringMultiple: 'popupsBlocked',
			allowString: 'popupBlockedAllow',
			allowStringAccessKey: 'popupBlockedAllowAccessKey',
			buttonText: 'notificationOptions',
			buttonAccessKey: 'notificationOptionsAccessKey',
			openString: 'openBlockedPopup',
		};

		this.runAsync(this.showNotification, this, 'abppa-notification-popup', win, uri, true, strings);
	},

	showLinkBlockedNotification: function(win, uri)
	{
		if(this.prefs.linkWarning == 0)
		{
			return;
		}
		else if((this.prefs.linkWarning == 2) && this.prefs.disableBrowserWarning)
		{
			return;
		}

		var strings = 
		{
			titleStringSingle: 'linkBlocked',
			titleStringMultiple: 'linksBlocked',
			allowString: 'linkBlockedAllow',
			allowStringAccessKey: 'linkBlockedAllowAccessKey',
			buttonText: 'notificationOptions',
			buttonAccessKey: 'notificationOptionsAccessKey',
			openString: 'openBlockedLink',
		};
		this.runAsync(this.showNotification, this, 'abppa-notification-link', win, uri, false, strings);
	},

	showAlertBlockedNotification: function(win)
	{		
		if(this.prefs.alertWarning == 0)
		{
			return;
		}
		else if((this.prefs.alertWarning == 2) && this.prefs.disableBrowserWarning)
		{
			return;
		}

		var strings = 
		{
			titleStringSingle: 'alertBlocked',
			titleStringMultiple: 'alertsBlocked',
			allowString: 'alertBlockedAllow',
			allowStringAccessKey: 'alertBlockedAllowAccessKey',
			buttonText: 'notificationOptions',
			buttonAccessKey: 'notificationOptionsAccessKey',
			openString: null,
		};
		this.runAsync(this.showNotification, this, 'abppa-notification-alert', win, null, true, strings);
	},

	showNotification: function(name, chromewin, uri, openInTab, strings)
	{
		var opener = this.createURI(chromewin.document.location);
		var browser = this.getDOMWindow(chromewin).gBrowser;
		var doc = browser.ownerDocument;
		var win = doc.defaultView;

		var strb = doc.getElementById('abppa-string-bundle');

		if(!strb) return;

        var notificationBox = browser.getNotificationBox();
		var oldNotification = notificationBox.getNotificationWithValue(name + '-blocked');
		var menu = doc.getElementById(name);

		if(oldNotification)
		{
			var val = this.getData(oldNotification, 'abppaCount') + 1;
			oldNotification.label = strb.getFormattedString(strings.titleStringMultiple, [val]);
			this.setData(oldNotification, 'abppaCount', val);
		}
		else
		{
			while (menu.hasChildNodes() && menu.lastChild.tagName != 'menuseparator')
			{
				menu.removeChild(menu.lastChild);
			}

			var buttons = [{
				accessKey: strb.getString(strings.buttonAccessKey),
				label: strb.getString(strings.buttonText),
				callback: null,
				popup: name
			}];

			var messageBarText = strb.getString(strings.titleStringSingle);
			const priority = notificationBox.PRIORITY_WARNING_MEDIUM;
			var newNotification = notificationBox.appendNotification(messageBarText, name + '-blocked', 
				                                                     "chrome://browser/skin/Info.png", priority, buttons);
			this.setData(newNotification, 'abppaCount', 1);
		}

		var allowItem = doc.getElementById(name + '-allow');
		var host = opener.host.replace(/^www\./, "");
		allowItem.setAttribute('label', strb.getFormattedString(strings.allowString, [host]));
		allowItem.setAttribute('accesskey', strb.getString(strings.allowStringAccessKey));
		allowItem.setAttribute('abppa-blocked-host', host);
		allowItem.setAttribute('abppa-action', 0);

		if(uri && (uri.scheme == 'http' || uri.scheme == 'https'))
		{
			var cleanUrl = uri.spec;

			if(cleanUrl.length > 128)
				cleanUrl = cleanUrl.substr(0, 128) + '...';

			var item = doc.createElement("menuitem");
			item.setAttribute("label", strb.getFormattedString(strings.openString, [cleanUrl]));
			item.addEventListener('click', function(){
				ABPPA.loadInBrowser(uri.spec, openInTab, win);
			}, false);
			
			menu.appendChild(item);
		}
	},

	handlePopup: function(win, uri, isTab, resolve, ret)
	{
		if(!this.checkURI(uri, win, isTab, resolve, ret) && !win.closed)
		{
			this.showPopupBlockedNotification(win.opener.top, uri);
			return false;
		}
		
		return true;
	},

	getDOMWindow: function(win)
	{
		var domwindow = win.QueryInterface(Ci.nsIInterfaceRequestor).
						getInterface(Ci.nsIWebNavigation).
						QueryInterface(Ci.nsIDocShellTreeItem).
						rootTreeItem.
						QueryInterface(Ci.nsIInterfaceRequestor).
						getInterface(Ci.nsIDOMWindow);


		// Firefox 3.x
		if(domwindow.wrappedJSObject)
		{
			domwindow = domwindow.wrappedJSObject;
		}

		return domwindow;
	},

	httpObserver:
	{
		observe: function(aSubject, aTopic, aData)
		{
			if(!ABPPA.enabled)
			{
				return;
			}

			switch(aTopic)
			{
				// Pop-up windows and regular links
				case 'http-on-modify-request':
					var channel = aSubject.QueryInterface(Ci.nsIChannel);

					if(!(channel instanceof Ci.nsIWritablePropertyBag))
					{
						return;
					}

					var requestData = null;

					if(ABPPA.previousRequest && ABPPA.previousRequest.uri == channel.URI)
					{
						channel.setProperty("abppaRequestData", ABPPA.previousRequest);
						requestData = ABPPA.previousRequest;
						ABPPA.previousRequest = null;

						// Add our listener to remove the data again once the request is done
						if (channel instanceof Ci.nsITraceableChannel)
						{
							new ABPPATraceableChannelCleanup(channel);
						}

						// Pop-ups are already handled on the first round
						if(requestData.popup)
							return;
					}
					else
					{
						try
						{
							requestData = channel.getProperty("abppaRequestData");
						}
						catch (e)
						{
							return;
						}
					}

					var cancel = false;

					// Pop-up
					if(requestData.popup)
					{
						var ret = {};
						if(!ABPPA.handlePopup(requestData.win, channel.URI, requestData.tab, false, ret))
						{
							ABPPA.closeWindow(requestData.win, ret.rule);
							cancel = true;
						}
					}
					// Regular link
					else
					{
						if(!ABPPA.checkHttpURI(requestData.win, channel.URI, requestData.origin))
						{
							cancel = true;
							ABPPA.showLinkBlockedNotification(requestData.win, channel.URI);
						}
					}

					if(cancel)
					{
						aSubject.loadFlags = Ci.nsICachingChannel.LOAD_ONLY_FROM_CACHE;
						aSubject.cancel(Cr.NS_ERROR_FAILURE);
					}
					break;

				// Alert dialogs
				case 'common-dialog-loaded':
				case 'tabmodal-dialog-loaded':
					if(aSubject && aSubject.args && !aSubject.closed)
					{
						var openerURI = null;
						var openerWin = null;
						if(aSubject.opener)
						{
							openerURI = ABPPA.createURI(aSubject.opener.top.document.location);
							openerWin = aSubject.opener.top;
						}
						else if(aSubject.previousSibling)
						{
							openerURI = aSubject.previousSibling.currentURI;
							openerWin = aSubject.previousSibling.contentWindow;
						}
						if(!openerURI || !openerWin || (openerURI.scheme !== 'http' && openerURI.scheme !== 'https'))
						{
							return;
						}
						var rule = ABPPA.getRule(openerURI.host, SiteRule.CHECK_DIALOGS);
						if(rule && rule.action == SiteRule.BLOCK)
						{
							aSubject.args.promptActive = false;
							aSubject.args.ok = true;
							aSubject.args.buttonNumClicked = 0;
							if(aSubject.close)
							{
								aSubject.close();
							}
							ABPPA.showAlertBlockedNotification(openerWin);
						}
					}
					break;
				default:
					return;
			}

			
		}
	},

	prefObserver: 
	{
		observe: function(subject, topic, data)
		{
			if (topic != 'nsPref:changed') return;
			
			ABPPA.loadPreferences(data);
		}
	},

	ownPrefObserver:
	{	
		observe: function(subject, topic, data)
		{
			if (topic != 'nsPref:changed') return;

			ABPPA.loadOwnPreferences(data);
		}
	},

	// ContentPolicy is used to handle regular links. It is only used to find out the
	// real referrer of the link. The blocking is done in httpObserver.
	contentPolicy:
	{
		classDescription: "Adblock Plus Pop-up Addon content policy",
		classID: Components.ID("{149a83e0-0e99-11df-8a39-0800200c9a66}"),
		contractID: "@adblockpluspopup.addon/policy;1",
		xpcom_categories: ["content-policy"],

		QueryInterface: XPCOMUtils.generateQI([Ci.nsIContentPolicy, Ci.nsIObserver,
			Ci.nsIFactory, Ci.nsISupportsWeakReference]),

		createInstance: function(outer, iid)
		{
			if (outer)
				throw Cr.NS_ERROR_NO_AGGREGATION;
			return this.QueryInterface(iid);
		},

		shouldLoad: function(contentType, contentLocation, requestOrigin, node, mimeTypeGuess, extra)
		{
			if(contentType == Ci.nsIContentPolicy.TYPE_DOCUMENT &&
			   ABPPA.enabled &&
			   node &&
			   requestOrigin &&
			   (requestOrigin.scheme === 'http' || requestOrigin.scheme === 'https'))
			{
				if(!node.contentWindow || !node.contentWindow.top)
				{
					return Ci.nsIContentPolicy.ACCEPT;
				}

				var win = node.contentWindow.top;

				// Handle regular link
				if(!ABPPA.isPopup(win))
				{
					if(ABPIntegration.installed)
					{
						ABPPA.previousRequest = {
							uri: contentLocation,
							origin: requestOrigin,
							browser: node,
							win: win,
							popup: false,
							tab: false
						};
					}
				}
				// Handle pop-up
				else if(win.opener.top)
				{
					var gbrowser = ABPPA.getDOMWindow(win.opener.top).gBrowser;

					// Fix some weird Hotmail logout stuff
					var winIndex = gbrowser.getBrowserIndexForDocument(win.document);
					var openerIndex = gbrowser.getBrowserIndexForDocument(win.opener.top.document);
					if(winIndex == openerIndex && winIndex != -1)
					{
						// Windows are same -> not actually a pop-up => handle as a regular link.
						// This happens for example when browser.link.open_newwindow is set to 1.
						if(ABPIntegration.installed)
						{
							ABPPA.previousRequest = {
								uri: contentLocation,
								origin: requestOrigin,
								browser: node,
								win: win,
								popup: false,
								tab: false
							};
						}

						return Ci.nsIContentPolicy.ACCEPT;
					}

					// If opener window was found in same browser as target window, window is a tab
					var isTab = (winIndex != -1);
					var resolve = true;

					// Check if this is actually a JavaScript or meta redirection
					var url = ABPPA.restoreWindowUrl(win);

					if(url)
					{
						resolve = false;
					}
					else
					{
						ABPPA.storeWindowData(win, contentLocation.spec, requestOrigin.spec);
					}

					// Disable pop-up handling for this window once user clicks it
					node.addEventListener('click', function(){
						ABPPA.onDocumentClicked(win);
					}, true);

					var ret = {};
					if(!ABPPA.handlePopup(win, contentLocation, isTab, resolve, ret))
					{
						// Don't close the last tab (and the whole browser window)
						if(!isTab || gbrowser.browsers.length > 1)
						{
							ABPPA.closeWindow(win, ret.rule);
						}

						return Ci.nsIContentPolicy.REJECT_REQUEST;
					}
					else if(ABPIntegration.installed)
					{
						var tmpUri = null;

						if(url && (tmpUri = ABPPA.createURI(url)))
						{
							requestOrigin = tmpUri;
						}

						ABPPA.previousRequest = {
							uri: contentLocation,
							origin: requestOrigin,
							browser: node,
							win: win,
							popup: true,
							tab: isTab
						};
					}
				}
			}

			return Ci.nsIContentPolicy.ACCEPT;
		},

		shouldProcess: function(contentType, contentLocation, requestOrigin, insecNode, mimeType, extra)
		{
			return Ci.nsIContentPolicy.ACCEPT;
		}
	}
	
};

function RedirectListener(uri)
{
	this.currentURI = uri;
	this.ready = false;
}

RedirectListener.prototype = 
{	
	onStartRequest: function (a, b) { },

	onDataAvailable: function (a, b, c, d, e) { },

	onStopRequest: function(aRequest, aContext, aStatus)
	{ 
		this.ready = true; 
	},

	onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) 
	{
		this.currentURI = aNewChannel.URI;
	},

	// Firefox 4
	asyncOnChannelRedirect: function(aOldChannel, aNewChannel, aFlags, redirectCallback)
	{
		this.onChannelRedirect(aOldChannel, aNewChannel, aFlags);
		redirectCallback.onRedirectVerifyCallback(0);
	},

	getInterface: function (aIID) 
	{
		try 
		{
			return this.QueryInterface(aIID);
		} 
		catch (e) 
		{
			throw Cr.NS_NOINTERFACE;
		}
	},
	
	onProgress : function (a, b, c, d) { },
	onStatus : function (a, b, c, d) { },
	onRedirect : function (a, b) { },
	
	QueryInterface: function(aIID) 
	{
		if (aIID.equals(Ci.nsISupports) ||
			aIID.equals(Ci.nsIInterfaceRequestor) ||
			aIID.equals(Ci.nsIChannelEventSink) ||
			aIID.equals(Ci.nsIProgressEventSink) ||
			aIID.equals(Ci.nsIHttpEventSink) ||
			aIID.equals(Ci.nsIStreamListener))
		{
			return this;
		}

		throw Cr.NS_NOINTERFACE;
	}

};

// Borrowed from Adblock Plus
// Author: Wladimir Palant
function ABPPATraceableChannelCleanup(request)
{
	// This has to run asynchronously due to bug 646370, nsHttpChannel triggers
	// http-on-modify-request observers before setting listener!
	ABPPA.runAsync(this.attach, this, request);
}
ABPPATraceableChannelCleanup.prototype =
{
	originalListener: null,

	attach: function(request)
	{
		if (request.isPending())
		{
			try
			{
				this.originalListener = request.setNewListener(this);
			}
			catch (e if e.result == Cr.NS_ERROR_NOT_IMPLEMENTED)
			{
				// Bug 646373 :-( Remove data even though this means that we won't be
				// able to block redirects.
				this.cleanup(request);
			}
		}
		else
			this.cleanup(request);
	},

	cleanup: function(request)
	{
		try
		{
			if (request instanceof Ci.nsIWritablePropertyBag)
			{
				request.deleteProperty("abppaRequestData");
			}
		}
		catch(e) {} // Ignore errors due to missing property
	},

	onStartRequest: function(request, context)
	{
		try
		{
			this.originalListener.onStartRequest(request, context);
		}
		catch (e if e instanceof Ci.nsIException)
		{
			request.cancel(e.result);
		}
	},

	onDataAvailable: function(request, context, inputStream, offset, count)
	{
		try
		{
			this.originalListener.onDataAvailable(request, context, inputStream, offset, count);
		}
		catch (e if e instanceof Ci.nsIException)
		{
			request.cancel(e.result);
		}
	},

	onStopRequest: function(request, context, statusCode)
	{
		try
		{
			this.originalListener.onStopRequest(request, context, statusCode);
		}
		catch (e if e instanceof Ci.nsIException)
		{
			// No point cancelling the channel when it is done already
		}
		finally
		{
			this.cleanup(request);
		}
	},

	QueryInterface: XPCOMUtils.generateQI([Ci.nsIStreamListener, Ci.nsIRequestObserver])
}

ABPPA.init();