/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ["RedirectionCache"];  

const RedirectionCache = 
{
	itemsObj: {},
	itemsArr: [],
	_maxSize: 0,

	set maxSize(value)
	{
		this._maxSize = value;
		this.cleanup();
	},

	get maxSize()
	{
		return this._maxSize;
	},

	get length()
	{
		return this.itemsArr.length;
	},

	clear: function()
	{
		this.itemsObj = {};
		this.itemsArr = [];
	},

	insert: function(source, target)
	{
		if(this.maxSize <= 0) return;

		if(this.itemsObj[source])
		{
			this.itemsObj[source] = target;
		}
		else
		{
			this.itemsObj[source] = target;
			this.itemsArr.push(source);
			this.cleanup();
		}
	},

	get: function(source)
	{
		if(this.itemsObj[source])
		{
			return this.itemsObj[source];
		}

		return null;
	},

	removeAt: function(index)
	{
		delete this.itemsObj[this.itemsArr[index]];
		this.itemsArr.splice(index, 1);
	},

	cleanup: function()
	{
		var toRemove = this.length - this.maxSize;
		
		if(toRemove > 0)
		{
			for(var i = 0; i < toRemove; i++)
			{
				this.removeAt(0);
			}
		}

	}
};